<?php

declare(strict_types=1);

namespace Drupal\entity_sync_session\SessionConfigurator\Oauth2;

// Entity Synchronization.
use Drupal\entity_sync_session\SessionConfigurator\PluginBase;
// Drupal core.
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
// Third-party libraries.
use KrystalCode\Api\Session\OAuth2\PasswordSessionManager;
use KrystalCode\Api\Session\SessionInterface;
use KrystalCode\Api\Session\SessionManagerInterface;
use KrystalCode\Api\Session\SessionStorageInterface;
use League\OAuth2\Client\Provider\GenericProvider;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for plugins using OAuth2 Resource Owner Password Credentials.
 *
 * Authentication configuration structure:
 * - client_id (string, required): The client ID.
 * - client_secret (string, required): The client secret.
 * - url_authorize (string, required): The authorization URL.
 * - url_access_token (string, required): The token URL.
 * - url_resource_owner_details (string, required): The resource owner URL.
 * - grant (array, required): An associative array containing settings related
 *   to the grant:
 *   - username (string, required): The username.
 *   - password (string, required): The password.
 *   - scope (string, optional): The scope.
 *
 * Supported configuration on top of what is supported by the base plugin:
 * - session_type_id (string, required): The ID used to store the session in the
 *   key value store.
 */
abstract class PasswordBase extends PluginBase implements
  ContainerFactoryPluginInterface {

  /**
   * The session manager.
   *
   * @var \KrystalCode\Api\Session\SessionManagerInterface
   */
  protected SessionManagerInterface|null $sessionManager = NULL;

  /**
   * Constructs a new PasswordBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \KrystalCode\Api\Session\SessionStorageInterface $sessionStorage
   *   The session storage.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected SessionStorageInterface $sessionStorage
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_sync_session.storage')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function setOptions(array $options): void {
    $this->sessionManager()->setOptions($options);
  }

  /**
   * {@inheritdoc}
   */
  public function getOptions(): array {
    return $this->sessionManager()->getOptions();
  }

  /**
   * {@inheritdoc}
   */
  public function connect(): SessionInterface {
    return $this->sessionManager()->connect();
  }

  /**
   * Instantiates and returns the session manager.
   *
   * @return \KrystalCode\Api\Session\SessionManagerInterface
   *   The session manager.
   */
  protected function sessionManager(): SessionManagerInterface {
    if ($this->sessionManager !== NULL) {
      return $this->sessionManager;
    }

    $authentication = &$this->configuration['authentication'];
    $options = [
      'grant' => [
        'username' => $authentication['username'],
        'password' => $authentication['password'],
      ],
      'type_id' => $this->configuration['session_type_id'],
    ];
    if (isset($authentication['scope'])) {
      $options['grant']['scope'] = $authentication['scope'];
    }

    $this->sessionManager = new PasswordSessionManager(
      new GenericProvider([
        'clientId' => $authentication['client_id'],
        'clientSecret' => $authentication['client_secret'],
        'urlAuthorize' => $authentication['url_authorize'],
        'urlAccessToken' => $authentication['url_access_token'],
        'urlResourceOwnerDetails' => $authentication['url_resource_owner_details'],
      ]),
      $this->sessionStorage,
      $options
    );

    return $this->sessionManager;
  }

}
